#include <stdio.h>
#include <stdlib.h>


#include "ImageBuffer.h"

#include <stk.h>
#include "sp_menu.h"
#include "sp_newgame.h"
#include "sp_about.h"



extern int Sp_width;
extern int Sp_height;

GLuint PlanetObj[MENUPLANETS];

// Color vectors
GLfloat grey[4] = {0.2, 0.2, 0.2, 1.0};
GLfloat white[4] = {1.0, 1.0, 1.0, 1.0};
GLfloat red[4] = { 1.0, 0.0, 0.0, 1.0 };
GLfloat blue[4] = { 0.0, 0.0, 1.0, 1.0 };
GLfloat teal[4] = { 0.0, 1.0, 0.8, 1.0 };

// Menu planets (buttons) struct and data
struct menuplanet_struct {
    float x_center;
    float y_center;
    float z_center;
    float radius;
    int rotation;
    char* texture;
    GLubyte active;
    char* menutext;
    float x;
    float y;
};

menuplanet_struct menuplanet[MENUPLANETS] = {
    {-0.5,  0.5,  0, 0.1, 0,"mercury.png",0, "Player"    ,100  ,320},
    { 0.5,  0.5,  0, 0.1, 0,"mars.png"   ,1, "New Game"  ,435 ,320},
    {-0.7,  0,    0, 0.1, 0,"earth.png"  ,0, "Options"   ,25 ,180},
    { 0.7,  0,    0, 0.1, 0,"jupiter.png",0, "High Score",490 ,180},
    {-0.5, -0.5,  0, 0.1, 0,"saturn.png" ,1, "About"     ,110  ,60},
    { 0.5, -0.5,  0, 0.1, 0,"uranus.png" ,1, "Quit"      ,460 ,60},
};


Sp_MenuPrincipal::Sp_MenuPrincipal(Stk_Looper *l)
        :Stk_Prog(l)
{


    float ambient[] = {0.1, 0.1, 0.1, 1.0};
    float diffuse[] = {0.5, 1.0, 1.0, 1.0};
    float position0[] = {-10.0, 10.0, 20.0, 0.0};
    float position1[] = {-10.0, 10.0, -20.0, 0.0};
    float front_mat_shininess[] = {60.0};
    float front_mat_specular[] = {0.2, 0.2, 0.2, 1.0};
    float front_mat_diffuse[] = {0.5, 0.28, 0.38, 1.0};
    float lmodel_ambient[] = {1.0, 1.0, 1.0, 1.0};
    float lmodel_twoside[] = {GL_FALSE};

    glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
    glLightfv(GL_LIGHT0, GL_POSITION, position0);
    glEnable(GL_LIGHT0);
  
    glLightfv(GL_LIGHT1, GL_AMBIENT, ambient);
    glLightfv(GL_LIGHT1, GL_DIFFUSE, diffuse);
    glLightfv(GL_LIGHT1, GL_POSITION, position1);
    glEnable(GL_LIGHT1);
  
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
    glLightModelfv(GL_LIGHT_MODEL_TWO_SIDE, lmodel_twoside);
    glEnable(GL_LIGHTING);
  
    glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, front_mat_shininess);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, front_mat_specular);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, front_mat_diffuse);
  
    glEnable(GL_DEPTH_TEST);
    glClearDepth(1.0);

    glShadeModel(GL_SMOOTH);

    InitPlanets();
        // On definit la projection
        // On utilise une projection perspective
    glMatrixMode(GL_PROJECTION);
    gluPerspective( /* angle de vue en degree */ 40.0,
                        /* coef de zoom */ 1.0,
                        /* Z near */ 1.0, /* Z far */ 10000.0);
        // On definit le placement de la camera
    glMatrixMode(GL_MODELVIEW);
    gluLookAt(0.0, 0.0, 2.5,      /* oeil a (0,0,5) */
              0.0, 0.0, -2.5,      /* centre  a (0,0,0) */
              0.0, 1.0, 0.);      /* le haut correspond au Y positif  */

    Sp_width=640;
    Sp_height=480;
    menuchoice=0;

    font=new Stk_FontTXF("default.txf");
    font->SetSize(15);

}


Sp_MenuPrincipal::~Sp_MenuPrincipal()
{
    delete font;
}


void Sp_MenuPrincipal::MessageReceived(Stk_Message *msg)
{
    int *valeur;
        // we have interest only in keyboard clavier
    if(msg->what!=KEY_EVENT){
        printf(" message joystick %d\n",msg->what);
        return;
    }
        // we have interest in keypressed message
    msg->FindInt("pressed",valeur);
    if(*valeur==0)
        return;
    msg->FindInt("keys",valeur);
    switch(*valeur){
        case K_UP:
            menuchoice -= 2;
            if (menuchoice < 0) {
                menuchoice = MENUPLANETS+menuchoice;
            }
            break;
        case K_DOWN:
            menuchoice+=2;
            if (menuchoice >= MENUPLANETS) {
                menuchoice = -MENUPLANETS+menuchoice;
            }
            break;
        case K_LEFT:
            menuchoice-=1;
            if (menuchoice < 0) {
                menuchoice = MENUPLANETS-1;
            }
            break;
        case K_RIGHT:
            menuchoice+=1;
            if (menuchoice >= MENUPLANETS) {
                menuchoice = 0;
            }
            break;
        case K_ENTER:
                // we analyze the choice
            if (menuplanet[menuchoice].active==1) {
                switch(menuchoice){
                    case 0: // Select player
                        break;
                    case 1: // Start game 
                        Sp_NewGame *myprog;
                        myprog=new Sp_NewGame(this);
                        break;
                    case 2: // Option menu
                        break;
                    case 3: // High scores
                        break;
                    case 4: // About Space Racer
                        Sp_About *about;
                        about=new Sp_About(this);
                        break;
                    case 5: // Quit game
                        Stk_QuitGL();
                        exit(0);
                        break;
                }
            }
            break;
        case K_QUIT: // Quit game
            Stk_QuitGL();
            exit(0);
            break;
    }
}

int Sp_MenuPrincipal::Loop()
{
        // On efface le color buffer
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glEnable(GL_LIGHTING);
    
    menuplanet[menuchoice].rotation += DZPLANETROTATION;
    if (menuplanet[menuchoice].rotation > 360) {
        menuplanet[menuchoice].rotation %= 360;
    }

    glPushMatrix();
        /* draw stuff here */
    DrawPlanets();
    glPopMatrix();

        // on flipe
    Stk_SwapBuffers();
    return 1;
}


void Sp_MenuPrincipal::DrawPlanets (void) {
    int planetcounter;
    float textoffset;
  

    Stk_Window *window;
    window=new Stk_Window(Sp_width,Sp_height);
  
        // Draw all planets
    for(planetcounter=0;planetcounter<MENUPLANETS;planetcounter++) {
      
        glPushMatrix();
        glTranslatef(menuplanet[planetcounter].x_center,
                     menuplanet[planetcounter].y_center,
                     menuplanet[planetcounter].z_center);
            // If the planet is active, then print the selected one .
        if (menuchoice == planetcounter) {
            glMaterialfv(GL_FRONT, GL_DIFFUSE, white);
        } else {
                // Inactive planets are grey
            glMaterialfv(GL_FRONT, GL_DIFFUSE, grey);
        }
      
        glRotatef(90,1,0,0);
        glRotatef(90,0,0,1);
        glRotatef(-menuplanet[planetcounter].rotation,0,0,1);
      
        glCallList(PlanetObj[planetcounter]);
      
        glRotatef(menuplanet[planetcounter].rotation,0,0,1);
        glRotatef(-90,0,0,1);
        glRotatef(-90,1,0,0);
      
            // Print text - first find offset = center of text, then print
            //textoffset = TEXTSCALE * 0.5 * gltext.CalculateStringWidth(menuplanet[planetcounter].menutext);
            //gltext.DrawText(-textoffset, -(menuplanet[planetcounter].radius + 0.06) , 0.1, menuplanet[planetcounter].menutext);
      
        Stk_Text *t1;
        t1=new Stk_Text(font,
                        menuplanet[planetcounter].menutext,
                        STK_LEFT,
                        menuplanet[planetcounter].x,
                        menuplanet[planetcounter].y,
                        window->width,window->height);
        window->Add_Widget(t1);
      
        window->Draw();
        glPopMatrix();
    }
    delete window;
}

void Sp_MenuPrincipal::InitPlanets ( void ) {

    int planetcounter;
  
    for(planetcounter=0;planetcounter<MENUPLANETS;planetcounter++) {

            // we read the image
        ImageBuffer img(menuplanet[planetcounter].texture);
        img.toRGBA();
            // we create the texture link
        glEnable(GL_TEXTURE_2D);
        glGenTextures ( 1,(unsigned int *) &num_tex ) ;
        glBindTexture(GL_TEXTURE_2D, num_tex);
        gluBuild2DMipmaps (GL_TEXTURE_2D,
                           4,             
                           img.getWidth(),img.getHeight(),
                           GL_RGBA,       
                           GL_UNSIGNED_BYTE,
                           (char*)img.getData());   

            // now we could generate a list
        PlanetObj[planetcounter] = glGenLists(1);
        glNewList(PlanetObj[planetcounter], GL_COMPILE);
      
        GLUquadricObj  *obj;
    
      
        obj = gluNewQuadric();
            // on la texture
        gluQuadricTexture(obj,GL_TRUE);
      
        glBindTexture(GL_TEXTURE_2D, num_tex);

        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT); 
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT); 
        gluSphere( obj, menuplanet[planetcounter].radius, PLANETSLICES, PLANETSTACKS);
        glDisable(GL_TEXTURE_2D);
  

        glEndList();
    }

}







