/*
  This file uses the GLX protocol to link X11 & OpenGL.
  This file was rewritten by Ellis (laurent_mallet@hotmail.com)
  using some source from Brian Paul.
*/



#include "stk_x11.h"



// global variable
Display *stk_x11_display;
Window   stk_x11_window;










static Window create_window(Display *display,
                          int width,
                          int height)
{
   int attrib[] = { GLX_RGBA,
                    GLX_RED_SIZE, 1,
                    GLX_GREEN_SIZE, 1,
                    GLX_BLUE_SIZE, 1,
                    GLX_DOUBLEBUFFER,
                    GLX_DEPTH_SIZE, 1, None };
                    
   
   int scrnum;
   XSetWindowAttributes attr;
   unsigned long mask;
   Window root;
   Window win;
   GLXContext ctx;
   XVisualInfo *visinfo;

   scrnum = DefaultScreen( display );
   root = RootWindow( display, scrnum );

   visinfo = glXChooseVisual( display, scrnum, attrib );
   if (!visinfo) {
       printf("Error: couldn't get an RGB, Double-buffered visual\n");
       exit(1);
   }

   /* window attributes */
   attr.background_pixel = 0;
   attr.border_pixel = 0;
   attr.colormap = XCreateColormap( display, root, visinfo->visual, AllocNone);
   attr.event_mask = ExposureMask | StructureNotifyMask | KeyPressMask | KeyReleaseMask |
       ButtonPressMask | ButtonMotionMask;

   mask = CWBackPixel | CWBorderPixel | CWColormap | CWEventMask;

   win = XCreateWindow( display, root, 0, 0, width, height,
		        0, visinfo->depth, InputOutput,
		        visinfo->visual, mask, &attr );

   ctx = glXCreateContext( display, visinfo, NULL, True );

   glXMakeCurrent( display, win, ctx );

   return win;
}



int Stk_SwapBuffers()
{
     glXSwapBuffers( stk_x11_display, stk_x11_window );
     return 0;
}



int Stk_EventLoop()
{
   XEvent event;

//   XNextEvent(stk_x11_display, &event);
   
   XCheckWindowEvent(stk_x11_display,stk_x11_window, 0xFFFFFFFF,&event);
   printf("event:%d\n",event.type);
   switch (event.type) {
   
       case Expose:
               // redraw( stk_x11_display, event.xany.window );
           break;
   }

   return 0;
}




int Stk_InitGL(int size_x,int size_y)
{
    stk_x11_display = XOpenDisplay(NULL);

    XAutoRepeatOff(stk_x11_display);
         
    stk_x11_window = create_window( stk_x11_display, size_x, size_y );
    
        // Once we have created the window, we draw it
    XMapWindow( stk_x11_display, stk_x11_window );
    return 0;
}


int Stk_QuitGL()
{
    XAutoRepeatOn(stk_x11_display);
    XFlush(stk_x11_display);
    return 0;
}
